function [x,w,y] = legquad(N)
%LEGQUAD Gauss-Legendre quadrature.
%   [X,W,Y] = LEGQUAD(N) returns points X and weights W (both row
%   vectors) for a Gauss quadrature with N points in the interval
%   (-1,1). The N-by-N matrix Y contains values of the Legendre
%   polynomials of order < N in all the quadrature points X. Each
%   column Y(:,i) contains the values of the polynomials evaluated at
%   X(i).
%
%   See also LEGEVAL.

% S. Engblom 2010-07-06 (Port to FLOW)
% S. Engblom 2009-07-10

% construct a symmetric tridiagonal matrix J
nn = 1:N;
b = nn./sqrt(4*nn.*nn-1);
b(end) = 0;
J = fsparse([[1 1:N-1]; [2:N N]],nn, ...
            [b([end 1:end-1]); b],[],'nosort');

% eigenvalues and -vectors
[y,x] = eig(full(J));

% the abscissas are the eigenvalues of J
x = diag(x)';

% the weights can be found from the first element of each eigenvector
w = 2*y(1,:).^2;

% the values of the polynomials themselves can be found from the
% eigenvectors (normalizing the first element to unity)
if nargout > 2
  y = tprod(y,1./y(1,:),[1 2],[3 2]);
  y = tprod(y,1./sqrt(1:2:2*N-1),[1 2],[3 1]);
end
