%EXPLOSION_REFINED Unstable/stable cases.

% S. Engblom 2010-10-06

% This script tests the unstable model for two values of psi_b
% below/above the stability limit. Then three stable models are tested
% for the unstable case. Plots in EXPLOSION_REFINEDPLOT.

% parameters as in EXPLOSIONS
Pe_phi = 1;
Pe_psi = 1;
Cn = 1/6;
Ex = 1;
%psic = 0.016;
%Pi = -(1+1/Ex)./(4*log(psic));
Pi = 0.1227;
psic = exp(-(1+1/Ex)/(4*Pi));

% determine suitable values of psib
% (1) unstable case
psib1 = 2*Pi*psic/(1-2*Pi); % approx. sufficient condition for instability

% approximate steady-state
psicfun = @(x)(exp(-1/4/Pi*((tanh(x/Cn).^2-1).^2+(1-tanh(x/Cn).^2)/Ex)));
psifun = @(x,y)(y./(y+psicfun(x)*(1-y)));

% solve for suitable initial data
psib1 = 0.5*psib1; % "unsharpness" factor 0.5 found by trial and error
[x,w] = legquad(100);
psib1init = w*psifun(x,psib1)'/2
psib1init = 0.012 % "rounder"

% (2) stable case
psib2 = 0.5*psib1; % "sufficient" factor 0.5 was found by trial and error
psib2init = w*psifun(x,psib2)'/2
psib2init = 0.006 % "rounder"

sol = cell(1,3);
t = [0 10];
u1 = {@(x)(tanh(x/Cn)) @(x)(psib1init*ones(size(x)))};
u2 = {@(x)(tanh(x/Cn)) @(x)(psib2init*ones(size(x)))};

% (1) unstable case
figure(1), clf
n = 1; fprintf('*** N = %d ***\n',n);
sol{n} = pde('w',[], ...
             'Pe_phi',Pe_phi, ...
             'Pe_psi',Pe_psi, ...
             'Cn',Cn, ...
             'Ex',Ex, ...
             'Pi',Pi, ...
             'u0',u1, ...
             'np',150, ...
             'nq',400, ...
             'filter',0.25, ...
             'plotcritical',true ...
             );
normw = sqrt(diag(sol{n}.cache.Mass));
normw = [normw; normw];
sol{n}.sol = ode1s(@pderhs,t,sol{n}.u0, ...
                   {'rtol',1e-6./normw,'atol',1e-8, ...
                    'errnorm2',@(z)(norm(z,inf)), ...
                    'nlinauto',1, ...
                    'reportfun',@pdereport}, ...
                   sol{n});

% (2) stable case
figure(2), clf
n = n+1; fprintf('*** N = %d ***\n',n);
sol{n} = pde('w',[], ...
             'Pe_phi',Pe_phi, ...
             'Pe_psi',Pe_psi, ...
             'Cn',Cn, ...
             'Ex',Ex, ...
             'Pi',Pi, ...
             'u0',u2, ...
             'np',150, ...
             'nq',400, ...
             'filter',0.25, ...
             'plotcritical',true ...
             );
normw = sqrt(diag(sol{n}.cache.Mass));
normw = [normw; normw];
sol{n}.sol = ode1s(@pderhs,t,sol{n}.u0, ...
                   {'rtol',1e-6./normw,'atol',1e-8, ...
                    'errnorm2',@(z)(norm(z,inf)), ...
                    'nlinauto',1, ...
                    'reportfun',@pdereport}, ...
                   sol{n});

% (3) stable model instead
figure(3), clf
n = n+1; fprintf('*** N = %d ***\n',n);
sol{n} = pde('w',[], ...
             'Pe_phi',Pe_phi, ...
             'Pe_psi',Pe_psi, ...
             'Cn',Cn, ...
             'Ex',Ex, ...
             'Fcase',1, ...
             'Pi',Pi, ...
             'u0',u1, ...
             'np',150, ...
             'nq',400, ...
             'filter',0.25 ...
             );
normw = sqrt(diag(sol{n}.cache.Mass));
normw = [normw; normw];
sol{n}.sol = ode1s(@pderhs,t,sol{n}.u0, ...
                   {'rtol',1e-6./normw,'atol',1e-8, ...
                    'errnorm2',@(z)(norm(z,inf)), ...
                    'nlinauto',1, ...
                    'reportfun',@pdereport}, ...
                   sol{n});

% (4) stable model #2
figure(4), clf
n = n+1; fprintf('*** N = %d ***\n',n);
sol{n} = pde('w',[], ...
             'Pe_phi',Pe_phi, ...
             'Pe_psi',Pe_psi, ...
             'Cn',Cn, ...
             'Ex',Ex, ...
             'Fcase',2, ...
             'Pi',Pi, ...
             'u0',u1, ...
             'np',150, ...
             'nq',400, ...
             'filter',0.25 ...
             );
normw = sqrt(diag(sol{n}.cache.Mass));
normw = [normw; normw];
sol{n}.sol = ode1s(@pderhs,t,sol{n}.u0, ...
                   {'rtol',1e-6./normw,'atol',1e-8, ...
                    'errnorm2',@(z)(norm(z,inf)), ...
                    'nlinauto',1, ...
                    'reportfun',@pdereport}, ...
                   sol{n});

% (5) stable model #3
figure(5), clf
n = n+1; fprintf('*** N = %d ***\n',n);
sol{n} = pde('w',[], ...
             'Pe_phi',Pe_phi, ...
             'Pe_psi',Pe_psi, ...
             'Cn',Cn, ...
             'Ex',Ex, ...
             'Fcase',3, ...
             'Pi',Pi, ...
             'u0',u1, ...
             'np',150, ...
             'nq',400, ...
             'filter',0.25 ...
             );
normw = sqrt(diag(sol{n}.cache.Mass));
normw = [normw; normw];
sol{n}.sol = ode1s(@pderhs,t,sol{n}.u0, ...
                   {'rtol',1e-6./normw,'atol',1e-8, ...
                    'errnorm2',@(z)(norm(z,inf)), ...
                    'nlinauto',1, ...
                    'reportfun',@pdereport}, ...
                   sol{n});

save data/explosion_refined sol
