%ISOTHERM Compute the adsorption isotherm.

% S. Engblom 2010-12-28 (Revision, now a total of 4 models)
% S. Engblom 2010-11-05 (Revision, new surfactant model)
% S. Engblom 2010-09-01

% This script tests a simple analytical prediction as to the
% equilibrium distribution of surfactant. Specifically, the loading of
% surfactant psi0 at the interface is considered as a function of the
% bulk concentration psib.

% caution since we might be overwriting files
if ~exist('Fname','var')
  error('Missing meta-data. Consider using the driver.');
end

% (1) parameters
Pe_phi = 1;
Pe_psi = 1;
Cn = 1/6;
Ex = 1;
psic = [0.0020 0.0056 0.016 0.035 0.075]; % range of interest
if expver == 0 % special case
  psic = 0.016;
end

% (2) analytical prediction
psib = logspace(-3,0,100);
phib = 1;
psi0a = tprod(psib,1./tsum(psib,psic,[3 2],[3 1]),[3 2],[1 2]);
figure(1); clf, h1 = gca;
loglog(h1,psib,psi0a,'b');
xlabel('\psi_b');
ylabel('\psi_0');
drawnow, hold on

% (3) numerical experiment
psib = logspace(-3,-1,7); % range of interest
if expver == 0 % special case
  psib = [1e-4 1e-3 1e-2];
end
phib = ones(size(psib));
Pi = frepmat(-(1+1/Ex)/4./log(psic),size(psib,2));
psibn = zeros(size(Pi,2),size(psib,2));
psi0n = zeros(size(psibn));
sol = cell(size(psibn));

% this ordering is more graphical than the obvious one
ij = [1 7 1 7 1 7 3 5 3 5 3 5 4 6 2 4 6 2 4 6 2 ...
      1 7 3 5 4 6 2 1 7 3 5 4 6 2; ...
      3 3 1 1 5 5 3 3 1 1 5 5 3 3 3 1 1 1 5 5 5 ...
      2 2 2 2 2 2 2 4 4 4 4 4 4 4];
if expver == 0
  ij = [1:3; 1 1 1]; % special case
end
for k = 1:size(ij,2)
  i = ij(1,k);
  j = ij(2,k);

  % approximate steady-state
  [phifun,psifun] = steady_state(Fcase,Cn,Ex,Pi(i,j),psib(i),true);
  psi0 = psifun(0);
  u0 = {phifun psifun};

  % solve
  figure(2), clf
  sol{j,i} = pde('w',[], ...
                 'Pe_phi',Pe_phi, ...
                 'Pe_psi',Pe_psi, ...
                 'Cn',Cn, ...
                 'Ex',Ex, ...
                 'Pi',Pi(i,j), ...
                 'Fcase',Fcase, ...
                 'u0',u0, ...
                 'np',300, ...
                 'nq',750, ...
                 'filter',0.25 ...
                 );
  t = [0 100];
  normw = sqrt(diag(sol{j,i}.cache.Mass));
  normw = [normw; normw];
  sol{j,i}.sol = ode1s(@pderhs,t,sol{j,i}.u0, ...
                       {'rtol',1e-6./normw,'atol',1e-8, ...
                      'errnorm2',@(z)(norm(z,inf)), ...
                      'nlinauto',1, ...
                      'reportfun',@pdereport}, ...
                       sol{j,i});
  % save first and last
  sol{j,i}.sol.x = sol{j,i}.sol.x([1 end]);
  sol{j,i}.sol.y = sol{j,i}.sol.y(:,[1 end]);

  % plot of first and final
  figure(3), clf
  x = sol{j,i}.cache.xx;
  [phi,psi] = pdeeval(sol{j,i},x,sol{j,i}.sol.x([1 end]));
  plot(x,phi(:,1),'b--',x,1/psi0*psi(:,1),'r--', ...
       x,phi(:,end),'b',x,1/psi0*psi(:,end),'r');
  drawnow;
    
  % extract isotherm
  x = linspace(0.95,1,50);
  x = [-x(end:-1:1) x];
  psi = sol{j,i}.sol.y(sol{j,i}.np+1:end,end);
  psibn(j,i) =  mean(pdeeval(psi,x)); % actual value of psib
  
  % measure the surfactant concentration at the interface
  x = linspace(-Cn*1e-4,Cn*1e-4,7);
  psi = sol{j,i}.sol.y(sol{j,i}.np+1:end,end);
  psi0n(j,i) =  mean(pdeeval(psi,x));

  % report value
  figure(1), hold on
  if sol{j,i}.sol.x(end) >= t(end)
    plot(h1,psibn(j,i),psi0n(j,i),'ko');
  else
    plot(h1,psib(i),psi0,'rx');
  end
  drawnow;
end

figure(1), axis([0.5e-3 1e0 1e-2 1e0]),

% pack...
for j = 1:size(sol,1)
  for i = 1:size(sol,2)
    if i > 1 || j > 1
      sol{j,i} = rmfield(sol{j,i},'cache');
    end
  end
end
if ~isempty(Fname)
  save(['data/' Fname],'sol','psibn','psi0n');
end
clear Fname % force caller/driver to make a new filename
