%VERIFICATION Convergence studies of simple cases.

% S. Engblom 2012-09-12

% case 1: pure Cahn-Hilliard flow compared with free-space traveling
% wave solution

% compare free-space BCs (analytical solution) vs. homogeneous
% Neumann (computed solution)
Cn = 1/10;
w = 0.25;
tspan = [-1 1]/2;
dfree = @(x)((1-tanh((x+w*tspan(end))/Cn).^2)/Cn);
max(abs(dfree([-1 1]))) % (this is a measure of the mismatch in BCs)

% intial data
u0 = {@(x)(tanh((x-w*tspan(1))/Cn)) @(x)(zeros(size(x)))};

% estimation of L^2-error
x = linspace(-1,1,101); dx = (x(2)-x(1))/(x(end)-x(1));
t = linspace(tspan(1),tspan(end),101); dt = (t(2)-t(1))/(t(end)-t(1));
phi_analytic = tanh(tsum(x,-w*t,[3 1],[3 2])/Cn);

% convergence experiment
N1 = 20:20:200;
TOL = [1e-4 1e-6 1e-8];
err1 = zeros(numel(TOL),numel(N1));
err1inf = zeros(numel(TOL),numel(N1));

for j = 1:numel(TOL)
  fprintf('*** j = %d\n',j);
  for i = 1:numel(N1)
    % build model
    sol = pde('w',w, ...
              'Cn',Cn, ...
              'u0',u0, ...
              'np',N1(i));

    % solve
    normw = sqrt(diag(sol.cache.Mass)); normw = [normw; normw];
    sol.sol = ode1s(sol.fun,tspan,sol.u0, ...
                    {'rtol',TOL(j)./normw,'atol',TOL(j)*1e-2, ...
                     'errnorm2',@(z)(norm(z,inf)), ...
                     'reportfun',[]}, ...
                    sol);

    % estimate the relative L^2-error
    phi = pdeeval(sol,x,t); 
    err1inf(j,i) = max(sqrt(sum((phi-phi_analytic).^2)*dx)./ ...
                       sqrt(sum(phi_analytic.^2)*dx));
  end
end
fprintf('Verification: case 1 done.\n');

% case 2: fully coupled case compared to highly resolved reference
% solution

% intial data
u0 = {@(x)(tanh((x-w*tspan(1))/Cn)) @(x)(1e-3*ones(size(x)))};

% reference solution
sol = pde('w',w, ...
          'Cn',Cn, ...
          'u0',u0, ...
          'Fcase',3, ...
          'np',400);
normw = sqrt(diag(sol.cache.Mass)); normw = [normw; normw];
sol.sol = ode1s(sol.fun,tspan,sol.u0, ...
                {'rtol',1e-10./normw,'atol',1e-12, ...
                 'errnorm2',@(z)(norm(z,inf)), ...
                 'reportfun',[]}, ...
                sol);
[phi_ref,psi_ref] = pdeeval(sol,x,t);

% convergence experiment
N2 = 100:25:300; % (this case is more challenging)
err2inf = zeros(numel(TOL),numel(N2));

for j = 1:numel(TOL)
  fprintf('*** j = %d\n',j);
  for i = 1:numel(N2)
    % build model
    sol = pde('w',w, ...
              'Cn',Cn, ...
              'u0',u0, ...
              'Fcase',3, ...
              'np',N2(i));

    % solve
    normw = sqrt(diag(sol.cache.Mass)); normw = [normw; normw];
    sol.sol = ode1s(sol.fun,tspan,sol.u0, ...
                    {'rtol',TOL(j)./normw,'atol',TOL(j)*1e-2, ...
                     'errnorm2',@(z)(norm(z,inf)), ...
                     'reportfun',[]}, ...
                    sol);

    % estimate the relative L^2-error
    [phi,psi] = pdeeval(sol,x,t);
    err2inf(j,i) = max(sqrt(sum((phi-phi_ref).^2)*dx)./ ...
                       sqrt(sum(phi_ref.^2)*dx)+ ...
                       sqrt(sum((psi-psi_ref).^2)*dx)./ ...
                       sqrt(sum(psi_ref.^2)*dx));
  end
end
fprintf('Verification: case 2 done.\n');

save data/verification N1 err1inf N2 err2inf
