function sol = pde(varargin)
%PDE Setup of a 4th order nonlinear PDE, Galerkin spectral method.
%   SOL = PDE(...) sets up a structure SOL with various fields
%   representing a certain PDE in two variables. Specifically,
%
%     phi_t + (w phi)_x = Pe_phi\(mu_phi)_xx,
%     psi_t + (w psi)_x = Pe_psi\[psi*(1-psi) mu_psi_x]_x,
%
%   where depending on the input variable Fcase,
%
%   Fcase = 0:
%     mu_phi = -phi + phi^3 - Cn^2/2 phi_xx + 
%              A + 1/(2Ex)psi phi,
%     mu_psi = Pi log(psi/(1-psi)) + 
%              B + 1/(4Ex)phi^2,
%
%   Fcase = 0:
%     A = Cn^2/2 [psi phi_xx+psi_x phi_x],
%     B = -Cn^2/4 (phi_x)^2,
%
%   Fcase = 1:
%     A = Cn^2/2 [psi phi_xx+psi_x phi_x],
%     B = -Cn^2/4 (phi_x)^2 - Cn^2/2 psi_xx - sigma/2 psi,
%
%   Fcase = 2:
%     A = phi psi/2,
%     B = phi^2/4,
%
%   Fcase = 3:
%     A = (1-phi^2) phi psi,
%     B = -(1-phi^2)^2/4.
%
%   Currently, only homogeneous Neumann conditions are supported. If
%   present, the velocity field should be a constant scalar.
%
%   Field        Value/{Default}        Description
%   -----------------------------------------------------------------------
%   u0           Cell-vector of two     Initial data for the two order 
%                functions {0 0.5}      parameters {phi psi}.
%
%   w            Function {[]}          Velocity (partially implemented).
%   Pe_phi       Scalar {1}             Peclet-numbers.
%   Pe_psi       Scalar {1}
%   Fcase        Scalar 0..3 {0}        Ginzburg-Landau energy (see above).
%   Cn           Scalar {0.1}           Cahn-number.
%   Ex           Scalar {1}             Bulk-solvability parameter.
%   Pi           Scalar {0.1}           Adsorption isotherm parameter.
%   sigma        Scalar {8*Pi}          Lateral interaction term (only
%                                       for Fcase = 1).
%
%   np           Scalar {50}            Number of spectral coefficients.
%   nq           Scalar {50}            Number of points in quadrature.
%   filter       Three scalars          Spectral filtering, see below.
%   clenshaw     Boolean {false}        Use Clenshaw-summation rather
%                                       than pre-evaluation.
%   lagproject   Boolean {false}        Use Lagrange multipliers when
%                                       projecting initial data to
%                                       enforce boundary conditions.
%   odemethod    Scalar {1}             Method of integration. The
%                                       only implemented case is 
%                                       currently 1 for backward Euler.
%   maxtime      Scalar {4*60*60}       Max computing time for
%                                       integration. PDEREPORT will
%                                       abort integration when the
%                                       estimated time to integrate
%                                       is more than maxtime (in
%                                       seconds).
%   plotcritical Boolean {false}        Plot points of instability
%                                       according to simplified
%                                       criterion. Only meaningful
%                                       for Fcase = 0.
%
%   The spectral filter implemented is an exponential one and consists
%   of three scalars [cutoff,order,alpha] with default values
%   [1,12,-log(eps)]. The first one is the cutoff of the filter, where
%   1 means no filtering and 0.25 means that the 25% lowest order
%   coefficients are unfiltered. The second parameter is the order of
%   the filter and the third parameter is the strength of the filter.
%
%   Reference:
%     [1] J. S. Hesthaven, S. Gottlieb and D. Gottlieb: "Spectral
%     Methods for Time-Dependent Problems", Cambridge University
%     Press (2007).
%
%   See also PDERHS, ODE1S, PDEREPORT, PDEEVAL, PDEMOVIE.

% S. Engblom 2012-09-12 (Partially implemented velocity)
% S. Engblom 2010-06-22 (Port to FLOW)
% S. Engblom 2007-03-09

% defaults
def = struct('w',[], ...
             'Pe_phi',1, ...
             'Pe_psi',1, ...
             'Fcase',0, ...
             'Cn',0.1, ...
             'Ex',1, ...
             'Pi',0.1, ...
             'sigma',[], ...
             'np',50, ...
             'nq',50, ...
             'filter',[1 12 -log(eps)], ... % [relative cutoff,order,alpha]
             'clenshaw',false, ...
             'lagproject',false, ...
             'odemethod',1, ... % 1: bwd Euler
             'maxtime',4*60*60, ...
             'plotcritical',false ...
             );
def.u0 = {@(x)(zeros(size(x))) @(x)(0.5*ones(size(x)))};

% special call: return defaults
if nargin == 0, sol = def; return; end

% merge defaults with actual inputs
if nargin == 1
  [sol,got] = parseopts(def,varargin{1});
else
  [sol,got] = parseopts(def,varargin);
end
if ~got.sigma
  % sigma less than or equal to 8*Pi ensures convexity of the logarithmic
  % double well free energy,
  %   Pi*[psi*log(psi)+(1-psi)*log(1-psi)]+sigma/4*psi*(1-psi),
  % and in this sense this choice makes Fcase = 0 and 1 comparable
  sol.sigma = 8*sol.Pi;
end

% order of the polynomials 0..np-1 (>= 2)
if sol.np < 2, error('Too low order of polynomials.'); end
Np = sol.np;

% precomputed constants for brevity
sol.Cn2 = sol.Cn^2;

switch sol.Fcase
  case {0,1,2,3}, ;
  otherwise, error('Only four cases allowed for property ''Fcase''. See help.');
end
% width of quadrature nq (>= np) and the quadrature itself
if got.np && ~got.nq
  sol.nq = Np;
elseif got.nq
  if sol.nq < Np, warning('Low order quadrature.'); end
end
Nq = sol.nq;
[sol.cache.xx,sol.cache.ww,sol.cache.utest] = legquad(Nq);
sol.cache.utest = sol.cache.utest(1:sol.np,:);
sol.cache.utestw = tprod(sol.cache.utest,sol.cache.ww,[1 2],[3 2]);

% filter, if any
if got.filter
  nfilt = numel(sol.filter);
  if nfilt > 3
    error('Filter must be defined by three scalars.');
  end
  def = [1 12 -log(eps)]; % (cutoff = 1 means no filtering)
  sol.filter = [sol.filter def(nfilt+1:end)];
end

% polynomials
j = [0.5 1:Np-1]';
sol.cache.PA = (2*j-1)./j*sol.cache.xx;
sol.cache.PB = (1-j)./j;
% prefer to trust [PA,PB] when property 'clenshaw' is true
if sol.clenshaw
  sol.cache.utest = clenshaw(sol.cache.PA,sol.cache.PB);
  sol.cache.utestw = tprod(sol.cache.utest,sol.cache.ww,[1 2],[3 2]);
end

% mass-matrix M
sol.cache.M = fsparse(1:Np,1:Np,2./(1:2:2*Np-1));

% differentiation-matrix in spectral space, u_x = D1\(D2*u)
b = [0 0 -1./[5:2:2*(Np)]; ...
     1 1./[3:2:2*(Np-1)] 1];
sol.cache.D1 = fsparse([[Np-1 Np 1:Np-2]; 1:Np],1:Np,b);
sol.cache.D2 = fsparse(1:Np-1,2:Np,ones(1,Np-1),Np);

% apply diagonal filter matrix
sol.cache.Mass = sol.cache.M; % field Mass holds the unfiltered mass-matrix
if sol.filter(1) < 1
  ncutoff = floor(Np*sol.filter(1));
  eta = (ncutoff+1:Np)/Np;
  % filter matrix saved for reference only (not used after this)
  sol.cache.F = fsparse(1:Np,1:Np,[ones(1,ncutoff) ...
                      exp(-sol.filter(3)* ...
                          ((eta-sol.filter(1))./(1-sol.filter(1))).^ ...
                          sol.filter(2))]);
  %sol.cache.D1 = sol.cache.D1/sol.cache.F; % *** (variant) 
  sol.cache.M = sol.cache.M/sol.cache.F;
end

% boundary operator, u([-1 1]) = B*u
sol.cache.B = clenshaw((2*j-1)./j*[-1 1],(1-j)./j)';
% (only used to check the initial boundary values)

% derivatives of polynomials at quadrature points
sol.cache.utestx = clenshaw(sol.cache.PA,sol.cache.PB, ...
                            full(sol.cache.D1\sol.cache.D2));
sol.cache.utestxw = tprod(sol.cache.utestx,sol.cache.ww,[1 2],[3 2]);

% project the initial data and check for consistency with boundary
% conditions
D1 = sol.cache.D1\sol.cache.D2;
D3 = sol.cache.D1\(sol.cache.D2*D1);
D3 = sol.cache.D1\(sol.cache.D2*D3);
D1 = sol.cache.B*D1;
D3 = sol.cache.B*D3;

u0 = sol.u0;
phi0rhs = tprod(sol.cache.utestw,u0{1}(sol.cache.xx), ...
                [1 -1],[2 -1]);
phi0 = sol.cache.M\phi0rhs;
if sol.lagproject
  % Lagrange multiplier matrix inverted through Sherman-Morrison
  U = [[[D1' D3']; -0.5*eye(4)] [zeros(Np,4); eye(4)]];
  V = U(:,[5:8 1:4]);
  UV = eye(8)+V'*[sol.cache.M\U(1:end-4,:); U(end-3:end,:)];
  U = U(1:end-4,:);
  V = V(1:end-4,:);
  
  % enforce Neumann boundary conditions by Lagrange multipliers
  dd = U*(UV\(V'*phi0));
  % (UV is often poorly conditioned and hence we keep the checks below)
  phi0 = sol.cache.M\(phi0rhs-dd);
end
Dphi0 = D1*phi0;
D3phi0 = D3*phi0;
if max(abs(Dphi0)) > 1e-6
  warning('pde:wBC',sprintf(['1st boundary condition in phi violated: ' ...
                      'Dphi(+-1) = [%f,%f]'],Dphi0(1),Dphi0(2)));
end
if max(abs(D3phi0)) > 1e-4
  warning('pde:wBC',sprintf(['2nd boundary condition in phi violated: ' ...
                      'D^3phi(+-1) = [%f,%f]'],D3phi0(1),D3phi0(2)));
end

% same for psi
psi0rhs = tprod(sol.cache.utestw,u0{2}(sol.cache.xx), ...
                [1 -1],[2 -1]);
psi0 = sol.cache.M\psi0rhs;
if sol.lagproject
  % 2nd order formulations 
  if sol.Fcase ~= 1
    U = [[D1'; -0.5*eye(2)] [zeros(Np,2); eye(2)]];
    V = U(:,[3:4 1:2]);
    UV = eye(4)+V'*[sol.cache.M\U(1:end-2,:); U(end-1:end,:)];
    U = U(1:end-2,:);
    V = V(1:end-2,:);
  end

  % (again: UV is often very poorly conditioned, checks below)
  dd = U*(UV\(V'*psi0));
  psi0 = sol.cache.M\(psi0rhs-dd);
end
Dpsi0 = D1*phi0;
D3psi0 = D3*psi0;
if max(abs(Dpsi0)) > 1e-6
  warning('pde:wBC',sprintf(['1st boundary condition in psi violated: ' ...
                      'Dpsi(+-1) = [%f,%f]'],Dpsi0(1),Dpsi0(2)));
end
if sol.Fcase == 1 && max(abs(D3psi0)) > 1e-4
  warning('pde:wBC',sprintf(['2nd boundary condition in psi violated: ' ...
                      'D^3psi(+-1) = [%f,%f]'],D3psi0(1),D3psi0(2)));
end
sol.u0 = [phi0; psi0];

% default right-hand side function
sol.fun = @pderhs;
