function [udimpl,udexpl] = pderhs(t,u,told,uold,sol)
%PDERHS Right-hand side of Galerkin spectral method.
%   [UDimpl,UDexpl] = PDERHS(T,U,Told,Uold,SOL) returns the
%   discretized right-hand side given the solution (Legendre
%   coefficients) U and the time T of a certain nonlinear PDE as
%   defined by the structure SOL. [Told,Uold] is the solution at a
%   previous time-step, and may be used to implement split-step
%   linearly implicit methods. UDimpl(UDexpl) is the
%   implicit(explicit) right-hand side.
%
%   See also PDE.

% S. Engblom 2012-09-12 (Partially implemented velocity)
% S. Engblom 2010-09-14 (Split-step method)
% S. Engblom 2010-06-22 (Port to FLOW)
% S. Engblom 2007-03-09

if sol.odemethod == 1 % backward Euler
  % split unknowns
  phi = u(1:sol.np);
  psi = u(sol.np+1:end);

  % coefficients of derivatives
  phix = sol.cache.D1\(sol.cache.D2*phi);
  psix = sol.cache.D1\(sol.cache.D2*psi);

  % must integrate to determine expansions of chemical potentials
  if ~sol.clenshaw
    % evaluate using stored coefficients
    phival = sol.cache.utest'*phi;
    phivalx = sol.cache.utest'*phix;
    psival = sol.cache.utest'*psi;
    psivalx = sol.cache.utest'*psix;
  else
    % evaluate using Clenshaw summation (slower, but with good
    % stability properties)
    phival = clenshaw(sol.cache.PA,sol.cache.PB,phi)';
    phivalx = clenshaw(sol.cache.PA,sol.cache.PB,phix)';
    psival = clenshaw(sol.cache.PA,sol.cache.PB,psi)';
    psivalx = clenshaw(sol.cache.PA,sol.cache.PB,psix)';
  end
  
  % auxiliary variable Phi/Psi
  Phirhs = -phival+phival.^3+0.5/sol.Ex*psival.*phival;
  switch sol.Fcase
    case 0
      Phi = sol.cache.M\(sol.cache.utestw*Phirhs+ ...
                         sol.cache.utestxw*0.5*sol.Cn2*((1-psival).*phivalx));
      Psirhs = -0.25*sol.Cn2*phivalx.^2+0.25/sol.Ex*phival.^2;
      Psi = sol.cache.M\(sol.cache.utestw*Psirhs);
    case 1
      Phi = sol.cache.M\(sol.cache.utestw*Phirhs+ ...
                         sol.cache.utestxw*0.5*sol.Cn2*((1-psival).*phivalx));
      % only in this case is psi governed by a 4th order equation:
      Psirhs = -0.25*sol.Cn2*phivalx.^2-0.5*sol.sigma*psival+ ...
               0.25/sol.Ex*phival.^2;
      Psi = sol.cache.M\(sol.cache.utestw*Psirhs+ ...
                         sol.cache.utestxw*0.5*sol.Cn2*psivalx);
    case 2
      Phirhs = Phirhs+0.5*psival.*phival;
      Phi = sol.cache.M\(sol.cache.utestw*Phirhs+ ...
                         sol.cache.utestxw*0.5*sol.Cn2*phivalx);
      Psirhs = 0.25*(1+1/sol.Ex)*phival.^2;
      Psi = sol.cache.M\(sol.cache.utestw*Psirhs);
   case 3
      Phirhs = Phirhs-(1-phival.^2).*phival.*psival;
      Phi = sol.cache.M\(sol.cache.utestw*Phirhs+ ...
                         sol.cache.utestxw*0.5*sol.Cn2*phivalx);
      Psirhs = -0.25*(1-phival.^2).^2+0.25/sol.Ex*phival.^2;
      Psi = sol.cache.M\(sol.cache.utestw*Psirhs);
  end

  % project
  Phix = sol.cache.D1\(sol.cache.D2*Phi);
  if ~sol.clenshaw
    Phivalx = sol.cache.utest'*Phix;
  else
    Phivalx = clenshaw(sol.cache.PA,sol.cache.PB,Phix)';
  end

  Psix = sol.cache.D1\(sol.cache.D2*Psi);
  if ~sol.clenshaw
    Psivalx = sol.Pi*psivalx+psival.*(1-psival).*(sol.cache.utest'*Psix);
  else
    Psivalx = sol.Pi*psivalx+psival.*(1-psival).* ...
              clenshaw(sol.cache.PA,sol.cache.PB,Psix)';
  end
    
  if isempty(sol.w)
    phit = -sol.cache.M\(1/sol.Pe_phi*(sol.cache.utestxw*Phivalx));
    psit = -sol.cache.M\(1/sol.Pe_psi*(sol.cache.utestxw*Psivalx));
  else
    % support for constant convection term
    phit = -sol.cache.M\(sol.cache.utestw*sol.w*phivalx+ ...
                         1/sol.Pe_phi*(sol.cache.utestxw*Phivalx));
    psit = -sol.cache.M\(sol.cache.utestw*sol.w*psivalx+ ...
                         1/sol.Pe_psi*(sol.cache.utestxw*Psivalx));
    % sketch of more general convection (term w * d/dx phi):
    %wphix = sol.cache.M\tprod(sol.w(t,sol.cache.xx), ...
    %                          tprod(sol.cache.utestw,phivalx,[1 2],1), ...
    %                          [2 -1],[1 -1]);
    %wpsix = sol.cache.M\tprod(sol.w(t,sol.cache.xx), ...
    %                          tprod(sol.cache.utestw,psivalx,[1 2],1), ...
    %                          [2 -1],[1 -1]);
  end
    
  % build rhs
  udimpl = [phit; psit];
  if nargout > 1
    udexpl = zeros(size(udimpl));
  end
else
  error('Unknown ODE method.');
end
