function E = energy(sol,t)
%ENERGY Evaluate energy.
%   E = ENERGY(SOL,T) evaluates the energy E of the solution in SOL
%   at times T. The output E is a vector of length numel(T).
%
%   E = ENERGY(SOL) evaluates the energy using the field SOL.sol.y directly.
%
%   No error-checking is performed.
%
%   See also MASS.

% S. Engblom 2010-08-24

% straightforward
if nargin > 1
  if strcmp(sol.sol.solver,'ode1s')
    u = interp1(sol.sol.x,sol.sol.y',t,'linear')';
  else
    u = deval(sol.sol,t);
  end
else
  u = sol.sol.y;
end

% evaluate in quadrature points
phi = u(1:sol.np,:);
psi = u(sol.np+1:end,:);
phix = sol.cache.D1\(sol.cache.D2*phi);

phival = phi'*sol.cache.utest;
phivalx = phix'*sol.cache.utest;
psival = psi'*sol.cache.utest;

% energy terms
Fphi = -0.5*phival.^2+0.25*phival.^4+0.25*sol.Cn2*phivalx.^2;
Fpsi = sol.Pi*(psival.*log(psival)+(1-psival).*log1p(psival));
F1 = -0.25*sol.Cn2*psival.*phivalx.^2;
Fex = 0.25/sol.Ex*psival.*phival.^2;
switch sol.Fcase
 case 1,
  psix = sol.cache.D1\(sol.cache.D2*psi);
  psivalx = psix'*sol.cache.utest;
  Fpsi = Fpsi+sol.sigma/4*psival.*(1-psival)+0.25*sol.Cn2*psivalx.^2;
 case 2,
  F1 = -0.25*psival.*(1-phival.^2);
 case 3,
  F1 = -0.25*psival.*(1-phival.^2).^2;
end

% integrate and final scaling
E = (Fphi+Fpsi+F1+Fex)*sol.cache.ww';
