function status = pdereport(t,u,s,varargin)
%PDEREPORT Report progress of PDE solver.
%   PDEREPORT is an alternative to REPORT that reports the progress of
%   the ODE time-stepper. PDEREPORT plots the solution of the PDE
%   instead of displaying a waitbar.
%
%   Unlike REPORT, PDEREPORT returns negative when the solution is
%   unphysical or will not be obtained sufficiently quickly. The
%   intergation is thus aborted (see ODE1S).
%
%   See also REPORT, PDEMOVIE.

% S. Engblom 2010-09-16

persistent T0 Tend tstart percent hax;

if isempty(s) && ~isempty(hax)
  sol = varargin{end};
  phi = pdeeval(u(1:sol.np),sol.cache.xx);
  psi = pdeeval(u(sol.np+1:end),sol.cache.xx);
  if sol.plotcritical
    % check for positive eigenvalue
    Dphi = sol.cache.D1\(sol.cache.D2*u(1:sol.np));
    Dphi = pdeeval(Dphi,sol.cache.xx);
    iproblem = find(sol.Pi <= sol.Cn2*Dphi.^2.*psi/2);
    cla(hax),
    plot(hax,sol.cache.xx(iproblem),0.5*max(abs(phi))/max(abs(psi))* ...
         psi(iproblem),'ko');
    hold on,
  end
  plot(hax,sol.cache.xx,phi,'b-', ...
       sol.cache.xx,0.5*max(abs(phi))/max(abs(psi))*psi,'r-');
  hold off;
  done = (t(end)-T0)/(Tend-T0); % fraction done so far...
  percent = round(done*100);    % ...in percent
  title(hax,sprintf('%d%% done',percent));
  drawnow;

  % avoid timing slow start-up moments
  if isempty(tstart)
    if done > 0
      tstart = tic;
    end
    ttotal = 0;
  else
    ttotal = toc(tstart); % total elapsed computing time
  end

  % ensure that psi is reasonably physical
  if sum(psi < 0) >= ceil(0.01*sol.nq) || ...
        -sum(psi(psi < 0)) > sum(psi(psi > 0)) || ...
        -min(psi) > max(psi)
    status = -1; % psi < 0
  elseif sum(psi > 1) >= ceil(0.01*sol.nq)
    status = -2; % psi > 1
  elseif norm(u(sol.np+1:floor(1.5*sol.np))) < 0.25*norm(u(ceil(1.5*sol.np):end))
    status = -3; % large high frequency content
  elseif ttotal > 30*60 && ttotal/done > sol.maxtime
    % (only trust the estimated total time after at least 30 minutes of
    % total computing time)
    status = -4; % solution will take too long to get
  else
    status = 0; % ok
  end
elseif strcmp(s,'init')
  T0 = t(1);
  Tend = t(end);
  tstart = [];
  percent = 0;
  hax = gca;
  cla(hax,'reset');
  title(hax,'0% done');
  drawnow;
  status = 0;
else
  status = 0;
end
