function [phi,psi] = steady_state(Fcase,Cn,Ex,Pi,psib,simple)
%STEADY_STATE Approximate steady-state of PDE.
%   [PHI,PSI] = STEADY_STATE(Fcase,Cn,Ex,Pi,psib,simple) determines
%   functions PHI and PSI which are approximations to the steady-state
%   of the equations in PDE with the stated parameters. The final
%   argument simple is a boolean. Use simple = true for the simpler
%   O(psib) approximation, and simple = false for a presumably more
%   accurate version.
%
%   No error-checking is performed.
%
%   See also PDE.

% S. Engblom 2011-01-13

if Fcase == 1
  warning('steady_state:wIncl','Only incomplete steady-state known.');
end

% determine phib and alpha
if simple % less detailed version, up to O(psib)
  phib = 1;
  alpha = 1;
else % detailed case with also O(psib)-terms included
  switch Fcase
   case {0 1}
    phib = sqrt(1-psib/(2*Ex));
    alpha = sqrt(1-psib);
   case 2
    phib = sqrt(1-(1+1/Ex)*psib/2);
    alpha = 1;
   case 3
    phib = sqrt((1-(1+1/(2*Ex))*psib)/(1-psib));
    alpha = sqrt(1-psib);
  end
end

% phi now known
phi = @(x)(phib*tanh(phib*x/Cn/alpha));
Dphi = @(x)(phib^2*sech(x/Cn/alpha).^2/Cn/alpha);

% determine psi
switch Fcase
 case 0, dB = @(x)(Cn^2*Dphi(x).^2);
 case 1, dB = @(x)(Cn^2*Dphi(x).^2); % *** more here
 case 2, dB = @(x)(phib^2-phi(x).^2);
 case {3 4}, dB = @(x)((phib^2-phi(x).^2).* ... 
                       (2-phib^2-phi(x).^2));
end
psic = @(x)(exp(-1/4/Pi*(dB(x)+1/Ex*(phib^2-phi(x).^2))));

if simple
  psi = @(x)(psib./(psib+psic(x)));
else
  psi = @(x)(psib./(psib+(1-psib)*psic(x)));
end
